import twitterag.user_follows as user_follows
import twitterag.tweet_lookup as tweet_lookup
import twitterag.user_profile as user_profile
import twitterag.likes as likes
import twitterag.config_tools as config_tools
import twitterag.exceptions.auth_except as AuthEX
from time import sleep
import re
import json
import requests
import os
import sys
import cmd



# Tweet Timeline Shell using Python standard library 'CMD'
#   
#   All shell commands are class methods prefixed with 'do_'. Example - do_help(), or do_profile()
#   All class methods/attributes are private unless they are a shell command method. 
#   Private attribute '__conf' is the configuration class instance. The vast majority of variables derive from this class
#   'AuthEX' is the shorthand for author defined exceptions imported from 'twitterag.exceptions' sub package
#   Besides 'do_profile', the class method 'retrieve_info()' is the aggregating method for this class. To see the flow of data and/or parameters, then start here
#



class tweet_timeline(cmd.Cmd):

    """Handle requests for tweet timelines"""

    prompt = "MODULE@INFO-timeline: "
    
    __conf = config_tools.ctools()
    
    __page_count = 0
    


    def do_timeline(self, arg):

        print("\nRunning {}\n".format(self.prompt))

        try:      

            pagination_bool = self.__conf.tweet_timeline_params["pagination"]["paginate?"]
            pagination_page_count = self.__conf.tweet_timeline_params["pagination"]["page_count"]
            read_from_file_bool = self.__conf.tweet_timeline_params["read_from_file?"]
            io_userid_readfile = self.__conf.file_IO["in"]["tweet_timeline"]["user_id_list"]
            user_id_string = self.__conf.tweet_timeline_params["user_id"]

            if read_from_file_bool == True:
                with open(io_userid_readfile, mode='r') as readfile:
                    for line in readfile:
                        if self.__input_file_check(line):
                            request = self.__retrieve_timeline(self.__url_build(line.strip()))
                            self.__dump_info(request)
                            self.__page_count = self.__page_count + 1
                            if pagination_bool == True:
                                while self.__page_count <= pagination_page_count:
                                    self.__conf.tweet_timeline_params["request_params"]["pagination_token"] = request["meta"]["next_token"]
                                    next_request = self.__retrieve_timeline(self.__url_build(line.strip()))
                                    request = next_request
                                    self.__dump_info(request)
                                    self.__page_count = self.__page_count + 1
                                self.__page_count = 0
                                return
                            elif pagination_bool == False:
                                self.__page_count = 0
                                return
                            else:
                                raise AuthEX.ParamTypeError(pagination_bool)
                        else:
                            print("\nNo data. Skipping line.\n")
                            pass
            elif read_from_file_bool == False:
                request = self.__retrieve_timeline(self.__url_build(user_id_string))
                self.__dump_info(request)
                self.__page_count = self.__page_count + 1
                if pagination_bool == True:
                    while self.__page_count <= pagination_page_count:
                        self.__conf.tweet_timeline_params["request_params"]["pagination_token"] = request["meta"]["next_token"]
                        next_request = self.__retrieve_timeline(self.__url_build(self.__conf.tweet_timeline_params["user_id"]))
                        request = next_request
                        self.__dump_info(request)
                        self.__page_count = self.__page_count + 1
                    self.__page_count = 0
                    return
                elif pagination_bool == False:
                    self.__page_count = 0
                    return
                else:
                    raise AuthEX.ParamTypeError(pagination_bool)
            else:
                raise AuthEX.ParamTypeError(read_from_file_bool)


        except FileNotFoundError:
            print("\nError: Readfile not found.\n")
            return

        except IsADirectoryError as dir_err:
            if dir_err.errno == 21:
                print("\nError: Directory found instead of readfile. file_IO path is probably empty.\n")
            return

        except KeyError as key_error:
            if "next_token" in key_error.args:
                pass
            else:
                print("\nConfig File Error: Bad key found in config file.\n")
                return

        except AuthEX.ParamTypeError as err:
            print("\nConfig File Error: Invalid parameter type: " + str(err) + ".\n")
            return




    def do_list(self, arg):
        
        request_params = json.dumps(self.__conf.tweet_timeline_params, indent=4, sort_keys=True)
        io_user_id_readfile = self.__conf.file_IO["in"]["tweet_timeline"]["user_id_list"]
        io_timeline_writefile = self.__conf.file_IO["out"]["tweet_timeline"]["tweet_timelines"]
        io_global = self.__conf.GLOBAL_FILE_PATH

        print("\n__Request__\n")
        print(str(request_params) + "\n")
        print("\n__Files__\n")
        print("    IN:")
        print("        " + io_user_id_readfile)
        print("    OUT:")
        print("        " + io_timeline_writefile)
        print("    GLOBAL:")
        print("        " + io_global)
        print("\n")

        return


    def do_set(self, arg):

        args_buff = str(arg)
        arg_list = args_buff.split()

        try:

            if arg_list[0] in ["request"]:
                if arg_list[1] in self.__conf.tweet_timeline_params:
                    if arg_list[1] in ["request_params"]:
                        if arg_list[2] in self.__conf.tweet_timeline_params["request_params"]:
                            if arg_list[3] in ["true", "false", "none"]:
                                if arg_list[3] == "true":
                                    self.__conf.tweet_timeline_params[arg_list[1]][arg_list[2]] = True
                                elif arg_list[3] == "false":
                                    self.__conf.tweet_timeline_params[arg_list[1]][arg_list[2]] = False
                                else:
                                    self.__conf.tweet_timeline_params[arg_list[1]][arg_list[2]] = None
                            else:
                                self.__conf.tweet_timeline_params[arg_list[1]][arg_list[2]] = arg_list[3]
                        else:
                            raise AuthEX.ShellArgError(arg_list[2])
                    elif arg_list[1] in ["pagination"]:
                        if arg_list[2] in self.__conf.tweet_timeline_params["pagination"]:
                            if arg_list[3] in ["true", "false", "none"]:
                                if arg_list[3] == "true":
                                    self.__conf.tweet_timeline_params[arg_list[1]][arg_list[2]] = True
                                elif arg_list[3] == "false":
                                    self.__conf.tweet_timeline_params[arg_list[1]][arg_list[2]] = False
                                else:
                                    self.__conf.tweet_timeline_params[arg_list[1]][arg_list[2]] = None
                            else:
                                self.__conf.tweet_timeline_params[arg_list[1]][arg_list[2]] = arg_list[3]
                        else:
                            raise AuthEX.ShellArgError(arg_list[2])
                    else:
                        if arg_list[2] in ["true", "false", "none"]:
                            if arg_list[2] == "true":
                                self.__conf.tweet_timeline_params[arg_list[1]] = True
                            elif arg_list[2] == "false":
                                self.__conf.tweet_timeline_params[arg_list[1]] = False
                            else:
                                self.__conf.tweet_timeline_params[arg_list[1]] = None
                        else:
                            self.__conf.tweet_timeline_params[arg_list[1]] = arg_list[2]
                else:
                    raise AuthEX.ShellArgError(arg_list[1])

            elif arg_list[0] in ["files"]:
                if arg_list[1] in ["global"]:
                    self.__conf.GLOBAL_FILE_PATH = arg_list[2]
                elif arg_list[1] in self.__conf.file_IO:
                    if arg_list[1] in ["out"]:
                        if arg_list[2] in self.__conf.file_IO[arg_list[1]]["tweet_timeline"]:
                            self.__conf.file_IO[arg_list[1]]["tweet_timeline"][arg_list[2]] = self.__conf.GLOBAL_FILE_PATH + arg_list[3]
                        else:
                            raise AuthEX.ShellArgError(arg_list[2])
                    else:
                        if arg_list[2] in self.__conf.file_IO["in"]["tweet_timeline"]:
                            self.__conf.file_IO["in"]["tweet_timeline"][arg_list[2]] = self.__conf.GLOBAL_FILE_PATH + arg_list[3] 
                        else:
                            raise AuthEX.ShellArgError(arg_list[2])
                else:
                    raise AuthEX.ShellArgError(arg_list[1])
            else:
                raise AuthEX.ShellArgError(arg_list[0])
        
            self.do_list(arg=None)
        
        except KeyError as key_error:
            print("\nError: Bad key in " + str(key_error.args))
            print("TIP: Config file corruption is possible with this error, but usually is due to a typo in args.\n")
            return

        except TypeError as t_err:
            print("\nError: Found \'None\' in: " + str(t_err.args) + ".\n")
            return

        except IndexError as inx_err:
            print("\nNot enough arguments, or too many for this functionality. Use \'help\' or \'?\' for usage.\n")
            return

        except AuthEX.ShellArgError as err:
            print("Error: Invalid shell argument specified: " + str(err) + ".\n")
            return

        return


    def do_help(self, arg):
        
        commands_list = [   
                                    "timeline = run the module with current parameters. No arguments.",
                                    "set [arg]* = where arg is either \'files\' or \'params\', following args are keys in a dictionary structure, and last arg is the value to be set.",
                                    "list [arg] = where arg is \'params\', \'commands\' or omitted completely."
                                    "help = print a detailed help page for this module.",
                                    "exit = terminate the entire program instance.",
                                    "main = direct to the main console.",
                                    "user = direct to the tweet user profile console.",
                                    "tweet = direct to the tweet lookup console.",
                                    "follows = direct to the follows console.",
                                    "likes = direct to the likes console."
                                ]
        print("\n__Commands__\n")

        for value in commands_list:
            print("    " + value + "\n")

        return


    def default(self, line: str) -> None:
        print("Invalid input...")
        sleep(1)
        return



    def emptyline(self):
        return
        


    def do_exit(self, arg):
        print("Terminating")
        sleep(2)
        sys.exit()



    def do_clear(self, arg):
        os.system("clear")
        return



    def do_tweet(self, arg):
        tweet_lookup_console = tweet_lookup.tweet_lookup()
        tweet_lookup_console.cmdloop()



    def do_follows(self, arg):
        user_follows_console = user_follows.follows()
        user_follows_console.cmdloop()



    def do_user(self, arg):
        user_profile_console = user_profile.user_profile()
        user_profile_console.cmdloop()



    def do_likes(self, arg):
        likes_console = likes.likes()
        likes_console.cmdloop()



    def __bearer_oauth(self, r):      

        auth_key_build = f"Bearer " + self.__conf.authorization["bearer_token"]
        r.headers["Authorization"] = auth_key_build 
        r.headers["User-Agent"] = "v2UserTweetsPython"

        return r



    def __param_engine(self):

        params =   {}
        param_table = self.__conf.tweet_timeline_params["request_params"]

        for key in param_table:
            if param_table[key] == None:
                pass
            else:
                params.update({key : param_table[key]})

        return params           



    def __url_build(self, user_id):
            
        url = "https://api.twitter.com/2/users/{}/tweets".format(user_id)

        return url



    def __retrieve_timeline(self, url):

        auth = self.__bearer_oauth
        params = self.__param_engine()

        request = requests.get(url, auth=auth, params=params)

        info_out = request.json()
        prettify = json.dumps(info_out, indent=4, sort_keys=True)

        if request.status_code != 200:

            print("Error(s)")
            print(request.status_code)
            print(request.content)
            return            
                
        else:
            if self.__conf.genopts["verbose?"]:
                print("\nrequest @ " + url + "\n")
                print(prettify + "\n")
                print("Response successful!\n")
            return info_out



    def __dump_info(self, json_object):

        try:

            writefile_path = self.__conf.file_IO["out"]["tweet_timeline"]["tweet_timelines"]
            file_extension = os.path.splitext(writefile_path)[1]

            if os.path.isfile(writefile_path):
                pass
            else:
                raise FileNotFoundError

            if file_extension in [".json"]:
                with open(writefile_path, mode='a') as writefile:
                    json.dump(json_object, writefile, indent=4, sort_keys=True)
                return    
            else:
                raise AuthEX.OutputFileError

        except AuthEX.OutputFileError:
            print("\nError: Writefile is not of .json type.\n")
            return
        
        except FileNotFoundError:
            print("\nError: Writefile not found.\n")
            return

        except IsADirectoryError:
            print("\nError: Writefile not found, found directory instead.\n")
            return  



    def __input_file_check(self, readline):

        regex_string = "[^\n\r\t\0]"
        regex_pattern = re.compile(regex_string)

        match_boolean = re.search(regex_pattern, readline)

        if match_boolean:
            return True
        elif match_boolean in [False, None]:
            return False


## Author created exceptions go here##

class ShellArgError(Exception):
    """exception for when a shell argument is not in scope"""
    
    def __init__(self, shell_argument):

        self.shell_arg = shell_argument

    def __str__(self) -> str:
        
        return(repr(self.shell_arg))


class ParamTypeError(Exception):
    """exception for invalid parameter (config) variable types"""
    
    def __init__(self, parameter):
        
        self.parameter = parameter

    def __str__(self):
        
        return(repr(self.parameter))    

class JsonFormatError(Exception):
    """exception for unexpected json response key"""
    pass

class OutputFileError(Exception):
    """exception for specified output files not being json"""
    pass





import twitterag.user_follows as user_follows
import twitterag.tweet_lookup as tweet_lookup
import twitterag.tweet_timeline as tweet_timeline
import twitterag.user_profile as user_profile
import twitterag.config_tools as config_tools
import twitterag.exceptions.auth_except as AuthEX
from time import sleep
import re
import json
import requests
import os
import sys
import cmd



# Likes Shell using Python standard library 'CMD'
#   
#   All shell commands are class methods prefixed with 'do_'. Example - do_help(), or do_profile()
#   All class methods/attributes are private unless they are a shell command method. 
#   Private attribute '__conf' is the configuration class instance. The vast majority of variables derive from this class
#   'AuthEX' is the shorthand for author defined exceptions imported from 'twitterag.exceptions' sub package
#   Besides 'do_profile', the class method 'retrieve_info()' is the aggregating method for this class. To see the flow of data and/or parameters, then start here
#



class likes(cmd.Cmd):

    """handles likes via tweet"""


    prompt = "MODULE@INFO-likes: "
    __conf = config_tools.ctools()
    __page_count = 0


    def do_liked(self, arg):

        print("\nRunning {}\n".format(self.prompt))

        try:

            read_from_file_bool = self.__conf.likes_params["read_from_file?"]
            pagination_bool = self.__conf.likes_params["pagination"]["paginate?"]
            pagination_page_count = self.__conf.likes_params["pagination"]["page_count"]
            user_id_string = self.__conf.likes_params["user_id"]
            io_liked_readfile = self.__conf.file_IO["in"]["likes"]["user_id_list"]

            if read_from_file_bool == True:
                with open(io_liked_readfile, mode='r') as readfile:
                    for line in readfile:
                        if self.__input_file_check(line):
                            request = self.__retrieve_info(self.__url_build(user_id=line.strip(), url_type="liked"), self.__bearer_oauth_liked, self.__param_engine("liked"))
                            self.__dump_info(request, "liked")
                            self.__page_count = self.__page_count + 1
                            if pagination_bool == True:
                                while self.__page_count <= pagination_page_count:
                                    self.__conf.likes_params["liked_request_params"]["pagination_token"] = request["meta"]["next_token"]
                                    next_request = self.__retrieve_info(self.__url_build(user_id=line.strip(), url_type="liked"), oauth_type=self.__bearer_oauth_liked, params_type=self.__param_engine("liked"))
                                    request = next_request
                                    self.__dump_info(request, "liked")
                                    self.__page_count = self.__page_count + 1
                                self.__page_count = 0
                                return
                            elif pagination_bool == False:
                                self.__page_count = 0
                                return
                            else:
                                raise AuthEX.ParamTypeError(pagination_bool)
                        else:
                            print("\nNo data. Skipping line.\n")
                            pass
            elif read_from_file_bool == False:
                request = self.__retrieve_info(self.__url_build(user_id=user_id_string, url_type="liked"), self.__bearer_oauth_liked, self.__param_engine("liked"))
                self.__dump_info(request, "liked")
                self.__page_count = self.__page_count + 1
                if pagination_bool == True:
                    while self.__page_count <= pagination_page_count:
                        self.__conf.likes_params["liked_request_params"]["pagination_token"] = request["meta"]["next_token"]
                        next_request = self.__retrieve_info(self.__url_build(user_id=user_id_string, url_type="liked"), oauth_type=self.__bearer_oauth_liked, params_type=self.__param_engine("liked"))
                        request = next_request
                        self.__dump_info(request, "liked")
                        self.__page_count = self.__page_count + 1
                    self.__page_count = 0
                    return
                elif pagination_bool == False:
                    self.__page_count = 0
                    return
                else:
                    raise AuthEX.ParamTypeError(pagination_bool)
            else:
                raise AuthEX.ParamTypeError(read_from_file_bool)

        except FileNotFoundError:
            print("\nError: Readfile not found.\n")
            return

        except IsADirectoryError as dir_err:
            if dir_err.errno == 21:
                print("\nError: Readfile not found, only a directory. file_IO path is probably empty.\n")
            return

        except KeyError as key_error:
            if "next_token" in key_error.args:
                pass
            else:
                print("\nConfig File Error: Bad key found in config file.\n")
                return

        except AuthEX.ParamTypeError as err:
            print("\nConfig File Error: Invalid parameter: " + str(err) + ".\n")
            return

        

    def do_liking(self, arg):

        print("\nRunning {}\n".format(self.prompt))

        try:

            read_from_file_bool = self.__conf.likes_params["read_from_file?"]
            pagination_bool = self.__conf.likes_params["pagination"]["paginate?"]
            pagination_page_count = self.__conf.likes_params["pagination"]["page_count"]
            tweet_id_string = self.__conf.likes_params["tweet_id"]
            io_liking_readfile = self.__conf.file_IO["in"]["likes"]["tweet_id_list"] 

            if read_from_file_bool == True:
                with open(io_liking_readfile, mode='r') as readfile:
                    for line in readfile:
                        request = self.__retrieve_info(self.__url_build(tweet_id=line.strip(), url_type="liking"), self.__bearer_oauth_liking, self.__param_engine("liking"))
                        self.__dump_info(request, "liking")
                        self.__page_count = self.__page_count + 1
                        if pagination_bool == True:
                            while self.__page_count <= pagination_page_count:
                                self.__conf.likes_params["liking_request_params"]["pagination_token"] = request["meta"]["next_token"]
                                next_request = self.__retrieve_info(self.__url_build(tweet_id=line.strip(), url_type="liking"), oauth_type=self.__bearer_oauth_liking, params_type=self.__param_engine("liking"))
                                request = next_request
                                self.__dump_info(request, "liking")
                                self.__page_count = self.__page_count + 1
                            self.__page_count = 0
                            return
                        elif pagination_bool == False:
                            self.__page_count = 0
                            return
                        else:
                            raise AuthEX.ParamTypeError(pagination_bool)
            elif read_from_file_bool == False:
                request = self.__retrieve_info(self.__url_build(tweet_id=tweet_id_string, url_type="liking"), self.__bearer_oauth_liking, self.__param_engine("liking"))
                self.__dump_info(request, "liking")
                self.__page_count = self.__page_count + 1
                if pagination_bool == True:
                    while self.__page_count <= pagination_page_count:
                        self.__conf.likes_params["liking_request_params"]["pagination_token"] = request["meta"]["next_token"]
                        next_request = self.__retrieve_info(self.__url_build(tweet_id=tweet_id_string, url_type="liking"), oauth_type=self.__bearer_oauth_liking, params_type=self.__param_engine("liked"))
                        request = next_request
                        self.__dump_info(request, "liking")
                        self.__page_count = self.__page_count + 1
                    self.__page_count = 0
                    return
                elif pagination_bool == False:
                    self.__page_count = 0
                    return
                else:
                    raise AuthEX.ParamTypeError(pagination_bool)
            else:
                raise AuthEX.ParamTypeError(read_from_file_bool)

        except FileNotFoundError:
            print("\nError: Readfile not found.\n")
            return

        except IsADirectoryError as dir_err:
            if dir_err.errno == 21:
                print("\nError: Readfile not found, only a directory. file_IO path is probably empty.\n")
            return

        except KeyError as key_error:
            print("\nConfig File Error: Bad key found in config file.\n")
            return

        except AuthEX.ParamTypeError as err:
            print("\nConfig File Error: Invalid parameter: " + str(err) + "\n")
            return
        
    
    
    def do_list(self, arg):

        request_params = json.dumps(self.__conf.likes_params, indent=4, sort_keys=True)
        io_liking_readfile = self.__conf.file_IO["in"]["likes"]["tweet_id_list"]
        io_liked_readfile = self.__conf.file_IO["in"]["likes"]["user_id_list"]
        io_liking_writefile = self.__conf.file_IO["out"]["likes"]["liking"]
        io_liked_writefile = self.__conf.file_IO["out"]["likes"]["liked"]
        io_global = self.__conf.GLOBAL_FILE_PATH

        print("\n__Request__\n")
        print(str(request_params) + "\n")
        print("\n__Files__\n")
        print("    IN:")
        print("        " + io_liking_readfile)
        print("        " + io_liked_readfile)
        print("    OUT:")
        print("        " + io_liking_writefile)
        print("        " + io_liked_writefile)
        print("    GLOBAL:")
        print("        " + io_global)
        print("\n")

        return                 



    def do_set(self, arg):
        
        args_buff = str(arg)
        arg_list = args_buff.split()

        try:

            if arg_list[0] in ["request"]:
                if arg_list[1] in self.__conf.likes_params:
                    if arg_list[1] in ["liking_request_params"]:
                        if arg_list[2] in self.__conf.likes_params["liking_request_params"]:
                            if arg_list[3] in ["true", "false", "none"]:
                                if arg_list[3] == "true":
                                    self.__conf.likes_params[arg_list[1]][arg_list[2]] = True
                                elif arg_list[3] == "false":
                                    self.__conf.likes_params[arg_list[1]][arg_list[2]] = False
                                else:
                                    self.__conf.likes_params[arg_list[1]][arg_list[2]] = None
                            else:
                                self.__conf.likes_params[arg_list[1]][arg_list[2]] = arg_list[3]
                        else:
                            raise AuthEX.ShellArgError(arg_list[2])
                    elif arg_list[1] in ["liked_request_params"]:
                        if arg_list[2] in self.__conf.likes_params["liked_request_params"]:
                            if arg_list[3] in ["true", "false", "none"]:
                                if arg_list[3] == "true":
                                    self.__conf.likes_params[arg_list[1]][arg_list[2]] = True
                                elif arg_list[3] == "false":
                                    self.__conf.likes_params[arg_list[1]][arg_list[2]] = False
                                else:
                                    self.__conf.likes_params[arg_list[1]][arg_list[2]] = None
                            else:
                                self.__conf.likes_params[arg_list[1]][arg_list[2]] = arg_list[3]
                        else:
                            raise AuthEX.ShellArgError(arg_list[2])
                    elif arg_list[1] in ["pagination"]:
                        if arg_list[2] in self.__conf.likes_params["pagination"]:
                            if arg_list[3] in ["true", "false", "none"]:
                                if arg_list[3] == "true":
                                    self.__conf.likes_params[arg_list[1]][arg_list[2]] = True
                                elif arg_list[3] == "false":
                                    self.__conf.likes_params[arg_list[1]][arg_list[2]] = False
                                else:
                                    self.__conf.likes_params[arg_list[1]][arg_list[2]] = None
                            else:
                                self.__conf.likes_params[arg_list[1]][arg_list[2]] = arg_list[3]
                        else:
                            raise AuthEX.ShellArgError(arg_list[2])
                    else:
                        if arg_list[2] in ["true", "false", "none"]:
                            if arg_list[2] == "true":
                                self.__conf.likes_params[arg_list[1]] = True
                            elif arg_list[2] == "false":
                                self.__conf.likes_params[arg_list[1]] = False
                            else:
                                self.__conf.likes_params[arg_list[1]] = None
                        else:
                            self.__conf.likes_params[arg_list[1]] = arg_list[2]
                else:
                    raise AuthEX.ShellArgError(arg_list[1])
            elif arg_list[0] in ["files"]:
                if arg_list[1] in ["global"]:
                    self.__conf.GLOBAL_FILE_PATH = arg_list[2]
                elif arg_list[1] in self.__conf.file_IO:
                    if arg_list[1] in ["out"]:
                        if arg_list[2] in self.__conf.file_IO[arg_list[1]]["likes"]:
                            self.__conf.file_IO[arg_list[1]]["likes"][arg_list[2]] = self.__conf.GLOBAL_FILE_PATH + arg_list[3]
                        else:
                                raise AuthEX.ShellArgError(arg_list[2])
                    else:
                        if arg_list[2] in self.__conf.file_IO["in"]["likes"]:
                            self.__conf.file_IO["in"]["likes"][arg_list[2]] = self.__conf.GLOBAL_FILE_PATH + arg_list[3] 
                        else:
                            raise AuthEX.ShellArgError(arg_list[2])
                else:
                    raise AuthEX.ShellArgError(arg_list[1])
            else:
                raise AuthEX.ShellArgError(arg_list[0])
        
            self.do_list(arg=None)

        except IndexError as inx_err:
            print("\nNot enough arguments, or too many for this functionality. Use \'help\' or \'?\' for usage.\n")
            return

        except KeyError as key_error:
            print("\nError: Bad key in " + str(key_error.args))
            print("TIP: Config file corruption is possible with this error, but usually is due to a typo in args.\n")
            return

        except TypeError as t_err:
            print("\nError: Found \'None\' in: " + str(t_err.args) + ".\n")
            return

        except AuthEX.ShellArgError as err:
            print("\nError: Invalid shell argument specified: " + str(err) + ".\n")
            return



    def do_help(self, arg):
        commands_list = [   
                                    "liking = run the module with current parameters, returning which user have liked a target tweet id",
                                    "liked = run the module with current parameters, returning which tweets a target user id has liked",
                                    "set [arg]* = where arg is either \'files\' or \'params\', following args are keys in a dictionary structure, and last arg is the value to be set.",
                                    "list [arg] = where arg is \'params\', \'commands\' or omitted completely."
                                    "help = print a detailed help page for this module.",
                                    "exit = terminate the entire program instance.",
                                    "main = direct to the main console.",
                                    "user = direct to the tweet user profile console.",
                                    "timeline = direct to the tweet timeline console.",
                                    "follows = direct to the follows console.",
                                    "lookup = direct to the tweet lookup console."
                                ]
        print("\n__Commands__\n")

        for value in commands_list:
            print("    " + value + "\n")

        return



    def default(self, line: str):
        print("Invalid input...")
        sleep(1)
        return



    def emptyline(self):
        return
        


    def do_clear(self, arg):
        os.system("clear")
        return



    def do_exit(self, arg):
        print("Terminating TAG...")
        sleep(2)
        sys.exit()



    def do_timeline(self, arg):
        tweet_timeline_console = tweet_timeline.tweet_timeline()
        tweet_timeline_console.cmdloop()



    def do_tweet(self, arg):
        tweet_lookup_console = tweet_lookup.tweet_lookup()
        tweet_lookup_console.cmdloop()



    def do_follows(self, arg):
        user_follows_console = user_follows.follows()
        user_follows_console.cmdloop()



    def do_user(self, arg):
        user_profile_console = user_profile.user_profile()
        user_profile_console.cmdloop()



    def __param_engine(self, param_type):

        params =    {

                    }

        liking_table = self.__conf.likes_params["liking_request_params"]
        liked_table = self.__conf.likes_params["liked_request_params"]

        if param_type == "liking":
            for key in liking_table:
                if liking_table[key] == None:
                    pass
                else:
                    params.update({key : liking_table[key]})
        elif param_type == "liked":
            for key in liked_table:
                if liked_table[key] == None:
                    pass
                else:
                    params.update({key : liked_table[key]})
        return params



    def __url_build(self, tweet_id="", user_id="", url_type=""): 

        if url_type == "liking":
            url = "https://api.twitter.com/2/tweets/{}/liking_users".format(tweet_id)
        elif url_type == "liked":
            url = "https://api.twitter.com/2/users/{}/liked_tweets".format(user_id)
            
        return url



    def __bearer_oauth_liking(self, r):  

        auth_key_build = f"Bearer " + self.__conf.authorization["bearer_token"]
        r.headers["Authorization"] = auth_key_build 
        r.headers["User-Agent"] = "v2LikingUsersPython"  
        
        return r

    def __bearer_oauth_liked(self, r):  

        auth_key_build = f"Bearer " + self.__conf.authorization["bearer_token"]
        r.headers["Authorization"] = auth_key_build 
        r.headers["User-Agent"] = "v2LikedTweetsPython"  
        
        return r


    def __retrieve_info(self, url, oauth_type, params_type): 
        
        request = requests.get(url, auth=oauth_type, params=params_type)

        info_out = request.json()

        if request.status_code != 200:
            
            print("Error(s): ")
            print(request.status_code)
            print(request.content)
            return
            
        elif request.status_code == 200:
            
            if self.__conf.genopts["verbose?"]:
                print("@" + url)
                print("Response successful!")

            return info_out



    def __dump_info(self, json_object, type):

        prettify = json.dumps(json_object, indent=4, sort_keys=True)

        if type == "liking":
            with open(self.__conf.file_IO["out"]["likes"]["liking"], mode='a') as writefile:
                json.dump(json_object, writefile, indent=4, sort_keys=True)
                if self.__conf.genopts["verbose?"]:
                    print(prettify)
                print("JSON successfully written!")
        elif type == "liked":
            with open(self.__conf.file_IO["out"]["likes"]["liked"], mode='a') as writefile:
                json.dump(json_object, writefile, indent=4, sort_keys=True)
                if self.__conf.genopts["verbose?"]:
                    print(prettify)
                print("JSON successfully written!")

        return


    def __dump_info(self, json_obj, type):

        try:

            prettify = json.dumps(json_obj, indent=4, sort_keys=True)

            writefile_path = ""


            if type == "liking":

                writefile_path = self.__conf.file_IO["out"]["likes"]["liking"]

            elif type == "liked":

                writefile_path = self.__conf.file_IO["out"]["likes"]["liked"]

            file_extension = os.path.splitext(writefile_path)[1]

            if os.path.isfile(writefile_path):
                pass
            else:
                raise FileNotFoundError

            if file_extension in [".json"]:
                with open(writefile_path, mode='a') as writefile:
                    json.dump(json_obj, writefile, indent=4, sort_keys=True)
                    if self.__conf.genopts["verbose?"]:
                        print(prettify)
                return    
            else:
                raise AuthEX.OutputFileError

        except AuthEX.OutputFileError:
            print("\nError: Writefile is not of .json type.\n")
            return
        
        except FileNotFoundError:
            print("\nError: Writefile not found.\n")
            return

        except IsADirectoryError:
            print("\nError: Writefile not found, found directory instead.\n")
            return  



    def __input_file_check(self, readline):

        regex_string = "[^\n\r\t\0]"
        regex_pattern = re.compile(regex_string)

        match_boolean = re.search(regex_pattern, readline)

        if match_boolean:
            return True
        elif match_boolean in [False, None]:
            return False    



#! usr/bin/env python
import twitterag.user_profile as user_profile
import twitterag.user_follows as user_follows
import twitterag.tweet_lookup as tweet_lookup
import twitterag.tweet_timeline as tweet_timeline
import twitterag.likes as likes
import twitterag.config_tools as config_tools
import os
import sys
import cmd
from time import sleep

version = "v.1.1.0\n"

class main_cli(cmd.Cmd):
    """main console command prompt"""
 
    user_profile_console = user_profile.user_profile()
    user_follows_console = user_follows.follows()
    tweet_lookup_console = tweet_lookup.tweet_lookup()
    tweet_timeline_console = tweet_timeline.tweet_timeline()
    likes_console = likes.likes()
    conf = config_tools.ctools()

    intro = version + "Main TAG console. Enter 'help' or '?' at anytime for usage."
    prompt = "MODULE@INFO:"

    def do_user(self, arg):
        self.user_profile_console.cmdloop()

    def do_timeline(self, arg):
        self.tweet_timeline_console.cmdloop()

    def do_tweet(self, arg):
        self.tweet_lookup_console.cmdloop()

    def do_follows(self, arg):
        self.user_follows_console.cmdloop()

    def do_likes(self, arg):
        self.likes_console.cmdloop()

    def do_help(self, arg):
        self.do_list(arg=None)
        print("\nFor in depth usage and information, see README.md in the source repo\n")
        return

    def default(self, line: str) -> None:
        print("Invalid input...")
        sleep(1)
        return

    def emptyline(self):
        return
        
    def do_exit(self, arg):
        print("Terminating")
        sleep(2)
        sys.exit()

    def do_clear(self, arg):
        os.system("clear")

    def do_list(self, arg):
        print("\nAvailable modules:")
        print("     user")
        print("     timeline")
        print("     tweet")
        print("     follows")
        print("     likes")



def main():
    cli = main_cli()
    cli.cmdloop()

main()



# Twitter-AG configuration file. See docs before editing! 
#       The vast majority of variables in TAG are attributes of this class.

class ctools:

        """configuration tools"""

        # dict for API authorization. These should be set and not touched again. 
        #       There is no access to this dict from the d0_set() function
        authorization = {
                                
                                "bearer_token" : None, # string
                                "api_key" : None, # string
                                "api_secret_key" : None, # string
                        }


        tweet_lookup_params =   {
                                        "read_from_file?" : False, # bool

                                        "tweet_id" : None, # string

                                        "request_params" :      {
                                                                       "expansions" : "attachments.poll_ids,attachments.media_keys,author_id,edit_history_tweet_ids,entities.mentions.username,geo.place_id,in_reply_to_user_id,referenced_tweets.id,referenced_tweets.id.author_id", # string
                                                                        "media.fields" : "duration_ms,height,media_key,preview_image_url,type,url,width,public_metrics,non_public_metrics,organic_metrics,promoted_metrics,alt_text,variants", # string
                                                                        "place.fields" : "contained_within,country,country_code,full_name,geo,id,name,place_type", # string
                                                                        "poll.fields" : "duration_minutes,end_datetime,id,options,voting_status", # string
                                                                        "tweet.fields" : "attachments,author_id,context_annotations,conversation_id,created_at,edit_controls,entities,geo,id,in_reply_to_user_id,lang,public_metrics,possibly_sensitive,referenced_tweets,reply_settings,source,text,withheld", # string 
                                                                }
                                }


        tweet_timeline_params = {

                                        "read_from_file?" : False, # bool

                                        "user_id" : None, # string

                                        "pagination" :  {
                                                                "paginate?" : False, # bool
                                                                "page_count" : 2 # int
                                                        },

                                        "request_params" :      {    
                                                                        "expansions" : "attachments.poll_ids,attachments.media_keys,author_id,edit_history_tweet_ids,entities.mentions.username,geo.place_id,in_reply_to_user_id,referenced_tweets.id,referenced_tweets.id.author_id", # string
                                                                        "max_results" : "5", # string
                                                                        "media.fields" : "duration_ms,height,media_key,preview_image_url,type,url,width,public_metrics,non_public_metrics,organic_metrics,promoted_metrics,alt_text,variants", # string
                                                                        "place.fields" : "contained_within,country,country_code,full_name,geo,id,name,place_type", # string
                                                                        "poll.fields" : "duration_minutes,end_datetime,id,options,voting_status", # string
                                                                        "tweet.fields" : "attachments,author_id,context_annotations,conversation_id,created_at,edit_controls,entities,geo,id,in_reply_to_user_id,lang,public_metrics,possibly_sensitive,referenced_tweets,reply_settings,source,text,withheld", # string
                                                                        "pagination_token" : None, # string
                                                                        "exclude" : None, # string
                                                                        "start_time" : None, # string
                                                                        "end_time" : None, # string
                                                                        "until_id" : None, # string
                                                                },
                                }       


        user_profile_params =   {       
                                        "usernames" : None, # string

                                        "user_id" : None, # string

                                        "search_by_username?": True, # bool

                                        "request_params" :      {
                                                                        "user.fields" : "description,id,public_metrics,created_at,entities,location,name,profile_image_url,protected,url,username,verified,withheld", # string
                                                                        "expansions" : "pinned_tweet_id", # string
                                                                },

                                        "read_from_file?" : False, # bool               
                                }


        user_follows_params =   {
                                        "read_from_file?" : False, # bool

                                        "user_id" : None, # bool

                                        "request_params" :      {
                                                                        "user.fields" : "description,id,public_metrics,created_at,entities,location,name,profile_image_url,protected,url,username,verified,withheld", # string
                                                                        "expansions" : "pinned_tweet_id", # string
                                                                        "max_results" : "5", # string
                                                                        "pagination_token" : None # string
                                                                },

                                        "pagination" :  {
                                                                "paginate?" : False, # bool
                                                                "page_count" : 2 # int
                                                        }
                                        
                                }


        likes_params =  {
                                "read_from_file?": False, # bool

                                "tweet_id" : None, # string

                                "user_id" : None, # string

                                "pagination" :  {
                                                        "paginate?" : False, # bool
                                                        "page_count" : 2, # int
                                                },

                                "liking_request_params" :       {
                                                                        "user.fields" : "description,id,public_metrics,created_at,entities,location,name,profile_image_url,protected,url,username,verified,withheld", # string
                                                                        "expansions" : "pinned_tweet_id", # string
                                                                        "max_results" : "5", # string
                                                                        "pagination_token" : None # string
                                                                },

                                "liked_request_params" :        {    
                                                                        "expansions" : "attachments.poll_ids,attachments.media_keys,author_id,edit_history_tweet_ids,entities.mentions.username,geo.place_id,in_reply_to_user_id,referenced_tweets.id,referenced_tweets.id.author_id", # string
                                                                        "max_results" : "5", # string
                                                                        "media.fields" : "duration_ms,height,media_key,preview_image_url,type,url,width,public_metrics,non_public_metrics,organic_metrics,promoted_metrics,alt_text,variants", # string
                                                                        "place.fields" : "contained_within,country,country_code,full_name,geo,id,name,place_type", # string
                                                                        "poll.fields" : "duration_minutes,end_datetime,id,options,voting_status", # string
                                                                        "tweet.fields" : "attachments,author_id,context_annotations,conversation_id,created_at,edit_controls,entities,geo,id,in_reply_to_user_id,lang,public_metrics,possibly_sensitive,referenced_tweets,reply_settings,source,text,withheld", # string
                                                                        "pagination_token" : None, # string
                                                                        "exclude" : None, # string
                                                                        "start_time" : None, # string
                                                                        "end_time" : None, # string
                                                                        "until_id" : None, # string
                                                                },
                        }

        GLOBAL_FILE_PATH = "" # string


        file_IO =       {
                                "out" : {

                                                "user_profile" :        {
                                                                                "user_profiles" : GLOBAL_FILE_PATH + "", # string
                                                                        },

                                                "tweet_timeline" :      {
                                                                                "tweet_timelines" : GLOBAL_FILE_PATH + "", # string
                                                                        },

                                                "user_follows" :        {
                                                                                "user_followers" : GLOBAL_FILE_PATH + "", # string

                                                                                "user_following" : GLOBAL_FILE_PATH + "", # string
                                                                        },

                                                "tweet_lookup" :        {
                                                                                "tweets" : GLOBAL_FILE_PATH + "", # string
                                                                        },

                                                "likes" :               {
                                                                                "liking" : GLOBAL_FILE_PATH + "", # string

                                                                                "liked" : GLOBAL_FILE_PATH + "", # string
                                                                        },
                                                
                                        },


                                "in" :  {
                                                "user_profile" :        {
                                                                                "username_list" : GLOBAL_FILE_PATH + "", # string

                                                                                "user_id_list" : GLOBAL_FILE_PATH + "", # string
                                                                        },

                                                "tweet_timeline" :      {
                                                                                "user_id_list" : GLOBAL_FILE_PATH + "", # string
                                                                        },

                                                "user_follows" :        {
                                                                                "user_id_list" : GLOBAL_FILE_PATH + "", # string
                                                                        },

                                                "tweet_lookup" :        {
                                                                                "tweet_id_list" : GLOBAL_FILE_PATH + "" # string
                                                                        },

                                                "likes" :               {
                                                                                "user_id_list" : GLOBAL_FILE_PATH + "", # string

                                                                                "tweet_id_list" : GLOBAL_FILE_PATH + "", # string
                                                                        },      
                                        }
                        }

        # genopts is reserved for general options that are infrequently used. 
        #       Currently, only verbose is an option. Reccomend that this is just left on (True)
        genopts =       {
                                "verbose?" : True # bool
                        }

